/*
 * IntrospectionPerformanceTest - Tests introspection on JavaBeans
 * Copyright (C) 2007 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.beanintrospect;

import org.apache.log4j.Logger;
import org.christianschenk.beanintrospect.beans.IntBean;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class PerformanceComparison {

	private static final Logger log = Logger.getLogger(PerformanceComparison.class);
	// it's just fair to initialize this class once
	private static final BeanIntrospector beanIntrospector = new BeanIntrospector();

	@Test
	public void testPerformance() {
		log.debug("Runs\tByHand\tIntrospector");
		final Runnable[] runners = new Runnable[] { byHandRunnable, beanIntrospectorRunnable };
		for (final int count : new int[] { 10, 100, 1000, 10000, 20000, 50000, 100000 }) {
//		for (int count = 1; count < 1000; count++) {
			final long times[] = new long[runners.length];
			int i = 0;
			for (final Runnable runner : runners) {
				long time = System.nanoTime();
				for (int runs = 0; runs < count; runs++) {
					runner.run();
				}
				time = System.nanoTime() - time;
				times[i++] = time / (1000 * 1000);
			}
			log.debug(count + "\t" + times[0] + "\t" + times[1] + "\t\t" + ((float) times[1] / (float) times[0]));
		}
	}

	private static final Runnable beanIntrospectorRunnable = new Runnable() {
		public void run() {
			final IntBean intBean1 = new IntBean(1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
			final IntBean intBean2 = new IntBean();
			beanIntrospector.fill(intBean2, intBean1);
			assertIntBeans(intBean1, intBean2);
		}
	};

	private static final Runnable byHandRunnable = new Runnable() {
		public void run() {
			final IntBean intBean1 = new IntBean(1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
			final IntBean intBean2 = new IntBean();
			intBean2.setInt1(intBean1.getInt1());
			intBean2.setInt2(intBean1.getInt2());
			intBean2.setInt3(intBean1.getInt3());
			intBean2.setInt4(intBean1.getInt4());
			intBean2.setInt5(intBean1.getInt5());
			intBean2.setInt6(intBean1.getInt6());
			intBean2.setInt7(intBean1.getInt7());
			intBean2.setInt8(intBean1.getInt8());
			intBean2.setInt9(intBean1.getInt9());
			intBean2.setInt10(intBean1.getInt10());
			assertIntBeans(intBean1, intBean2);
		}
	};

	private static final void assertIntBeans(final IntBean intBean1, final IntBean intBean2) {
		assertEquals(1, intBean1.getInt1());
		assertEquals(1, intBean2.getInt1());
		assertEquals(2, intBean1.getInt2());
		assertEquals(2, intBean2.getInt2());
		assertEquals(3, intBean1.getInt3());
		assertEquals(3, intBean2.getInt3());
		assertEquals(4, intBean1.getInt4());
		assertEquals(4, intBean2.getInt4());
		assertEquals(5, intBean1.getInt5());
		assertEquals(5, intBean2.getInt5());
		assertEquals(6, intBean1.getInt6());
		assertEquals(6, intBean2.getInt6());
		assertEquals(7, intBean1.getInt7());
		assertEquals(7, intBean2.getInt7());
		assertEquals(8, intBean1.getInt8());
		assertEquals(8, intBean2.getInt8());
		assertEquals(9, intBean1.getInt9());
		assertEquals(9, intBean2.getInt9());
		assertEquals(10, intBean1.getInt10());
		assertEquals(10, intBean2.getInt10());
	}
}